"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = lazyObject;

var _AOPProxy = _interopRequireDefault(require("../AOPProxy"));

var _pointcuts = require("../pointcuts/pointcuts");

var _escapeStringRegexp = _interopRequireDefault(require("escape-string-regexp"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/**
 * Builtin function using AOPProxy's API to implement lazy loading for objects.
 * @param {Object} target A target object to lazy load. Lazy loading in this case means
 * deferring an operation on the object when it is used for the very first time by the
 * client code.
 * @param {Object} options The options of the lazy loading.
 * @param {Function} onceCallback The callback to execute only once before performing the very
 * first operation on the given "target" when the given "target" is used for the very first time.
 * This callback is mandatory (otherwise it doesn't make sense to use "lazy()" at all)
 * and allows e.g. to perform a heavy initialization operation lazily on the underlying target
 * object.
 * @param {Object} [options.ignoredTypes] The types which prototypes to ignore when going up
 * the prototype chain in order to identify which prototypes to intercept when lazy loading.
 * By default, only the properties set on the "Object" prototype will be ignored.
 * This option can be used e.g. to avoid the performance of a heavy lazy initialization
 * when calling the methods of a parent class of the given "target" object.
 * @param {boolean} [options.interceptConstructor] Whether or not to intercept the
 * getting/setting of the "constructor" property. Defaults to "false", meaning that
 * when getting/setting "target.constructor", this operation will not trigger the underlying
 * lazy loading.
 * @param {string[]} [options.propertiesToIgnore] Property names or method names to ignore for
 * lazy loading.
 * @param {Array} [options.pointcuts] Array of pointcut shorthands to use for interception and
 * lazy loading. By default, only "call", "get" and "set" pointcuts will be used. Other pointcut
 * shorthands (e.g. "apply", "construct") will be ignored, if passed.
 * @return {Proxy} A new proxy object for the given target which supports this builtin feature.
 */
function lazyObject(target) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      onceCallback = _ref.onceCallback,
      _ref$ignoredTypes = _ref.ignoredTypes,
      ignoredTypes = _ref$ignoredTypes === void 0 ? [Object] : _ref$ignoredTypes,
      _ref$interceptConstru = _ref.interceptConstructor,
      interceptConstructor = _ref$interceptConstru === void 0 ? false : _ref$interceptConstru,
      _ref$propertiesToIgno = _ref.propertiesToIgnore,
      propertiesToIgnore = _ref$propertiesToIgno === void 0 ? [] : _ref$propertiesToIgno,
      _ref$pointcuts = _ref.pointcuts,
      pointcuts = _ref$pointcuts === void 0 ? [_pointcuts.call, _pointcuts.get, _pointcuts.set] : _ref$pointcuts;

  var hasCall = false,
      hasGet = false,
      hasSet = false;
  var rules = [];
  var propertiesMap = {};
  var constructorsToIgnore = ignoredTypes.map(function (Type) {
    return Type.prototype.constructor;
  });
  var current = target;
  var currentIsProto = false;

  while (current) {
    var _constructor = current.constructor;

    if (!currentIsProto || !constructorsToIgnore.includes(_constructor)) {
      var _properties = Reflect.ownKeys(current);

      _properties.map(function (property) {
        return propertiesMap[property] = {
          isMethod: property !== "constructor" && typeof target[property] === "function"
        };
      });
    }

    current = Object.getPrototypeOf(current);
    currentIsProto = true;
  }

  propertiesToIgnore.map(function (propertyToIgnore) {
    return delete propertiesMap[propertyToIgnore];
  });
  !interceptConstructor && delete propertiesMap["constructor"];
  pointcuts.map(function (pointcut) {
    if (pointcut === _pointcuts.call) {
      hasCall = true;
    } else if (pointcut === _pointcuts.get) {
      hasGet = true;
    } else if (pointcut === _pointcuts.set) {
      hasSet = true;
    }
  });
  var properties = Object.keys(propertiesMap);
  var firstOperation = true;

  var callback = function callback() {
    if (firstOperation) {
      firstOperation = false;

      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      onceCallback.apply(this, args);
    }
  };

  var escape = function escape(names) {
    return names.map(_escapeStringRegexp["default"]);
  };

  var regex = function regex(escapedNames) {
    return new RegExp("^(".concat(escapedNames.join("|"), ")$"));
  };

  var addRule = function addRule(regex) {
    return function (pointcut) {
      return rules.push([regex, pointcut().before(callback)]);
    };
  };

  if (hasCall) {
    var methodNames = properties.filter(function (property) {
      return propertiesMap[property].isMethod;
    });
    var escapedMethodNames = escape(methodNames);
    addRule(regex(escapedMethodNames))(_pointcuts.call);
  }

  if (hasGet || hasSet) {
    var propertyNames = properties.filter(function (property) {
      return !propertiesMap[property].isMethod;
    });
    var escapedPropertyNames = escape(propertyNames);
    hasGet && addRule(regex(escapedPropertyNames))(_pointcuts.get);
    hasSet && addRule(regex(escapedPropertyNames))(_pointcuts.set);
  }

  var proxyTarget = (0, _AOPProxy["default"])(target, rules);
  return proxyTarget;
}
//# sourceMappingURL=lazyObject.js.map