"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = throwErrorForUnknownProperty;

var _AOPProxy = _interopRequireDefault(require("../AOPProxy"));

var _pointcuts = require("../pointcuts/pointcuts");

var _strings = require("../../misc/strings");

var _objects = require("../../object-handling/objects");

var _excluded = ["dynamic", "errorToThrow"];

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

/**
 * Builtin function using AOPProxy's API to throw an error when an unknown property is accessed on the given target
 * (both for read and write operations).
 * @param {Object} target The target object.
 * @param {Object} options Options.
 * @param {boolean} [options.dynamic] Whether or not to check against the current properties of
 * the target on each property access (dynamic, default). If set to false, then only the
 * properties of the target object at the time this builtin function is called will be
 * treated as known and used for the check when determining if a property is unknown.
 * @param {?Error|Function} [options.errorToThrow] The error to throw or null (default) or a
 * function returning an error instance to throw. If null, a default error will be thrown when
 * trying to access an unknown property. If a function is given, the function will receive an
 * object with the following properties as parameter:
 * - target (Object): The target object;
 * - propertyName (string): The name of the unknown property;
 * - targetTypeToStr (string): The type of the target (e.g. "[object Object]");
 * - defaultErrorMessage (string): A default error message which can be used for the custom
 * error;
 * @return {Proxy} A new proxy object for the given target which supports this builtin feature.
 */
function throwErrorForUnknownProperty(target) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$dynamic = _ref.dynamic,
      dynamic = _ref$dynamic === void 0 ? true : _ref$dynamic,
      _ref$errorToThrow = _ref.errorToThrow,
      errorToThrow = _ref$errorToThrow === void 0 ? null : _ref$errorToThrow,
      prototypeChainPropertiesOptions = _objectWithoutProperties(_ref, _excluded);

  var regex = /.?/;
  var map = {};

  var mapProperties = function mapProperties() {
    var props = (0, _objects.prototypeChainProperties)(target, prototypeChainPropertiesOptions);
    map = props.reduce(function (acc, prop) {
      return !(acc[prop] = true) || acc;
    }, {});
  };

  !dynamic && mapProperties();

  var assertFn = function assertFn() {
    var propertyName = this.property;
    var targetTypeToStr = (0, _strings.typeToStr)(target);
    var defaultErrorMessage = "Accessing unknown property \"".concat(propertyName, "\" of object ").concat(targetTypeToStr, ".");
    dynamic && mapProperties();

    if (!map[propertyName]) {
      errorToThrow = errorToThrow ? typeof errorToThrow === "function" ? errorToThrow({
        target: target,
        propertyName: propertyName,
        targetTypeToStr: targetTypeToStr,
        defaultErrorMessage: defaultErrorMessage
      }) : errorToThrow : new Error(defaultErrorMessage);
      throw errorToThrow;
    }
  };

  var proxyTarget = (0, _AOPProxy["default"])(target, [[regex, (0, _pointcuts.get)().before(assertFn)], [regex, (0, _pointcuts.set)().before(assertFn)]]);
  return proxyTarget;
}
//# sourceMappingURL=throwErrorForUnknownProperty.js.map