"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.colorFromString = colorFromString;
exports.getLuminance = getLuminance;
exports.hexToRgb = hexToRgb;
exports.intToHex = intToHex;
exports.rgbToHex = rgbToHex;

var _hash = require("../maths/hash");

var _math = require("../maths/math");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

/**
 * Get the luminance value of a color in HEX format.
 * @param {string} hexColor a colour in hex format. leading '#' will be removed if present.
 * @param {number} precision sig figs to round the result to.
 * @returns {number} the luminance.
 */
function getLuminance(hexColor) {
  var precision = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 2;
  if (!hexColor) return 0;
  hexColor = hexColor.replace(/^#/, "");
  var c = parseInt(hexColor, 16);
  var r = (c & 0xff0000) >> 16;
  var g = (c & 0x00ff00) >> 8;
  var b = c & 0x0000ff;
  var luminance = 0.299 * r + 0.587 * g + 0.114 * b;
  return (0, _math.round)(luminance, precision);
}
/**
 * Convert a number to a HEX string that can be used as a color.
 * @param {number} i a number between 0 and Number.MAX_SAFE_INTEGER.
 * @returns {string} A valid HEX color.
 */


function intToHex(i) {
  var c = (i & 0x00ffffff).toString(16).toUpperCase();
  return "00000".substring(0, 6 - c.length) + c;
}
/**
 * Convert a string to a HEX string that can be used as a color.
 * @param {string} str The string to convert.
 * @returns {string} A valid HEX color.
 */


function colorFromString(str) {
  var hash = (0, _hash.hashString)(str);
  return intToHex(hash);
}

var hexCharacters = "a-f\\d";
var match3or4Hex = "#?[".concat(hexCharacters, "]{3}[").concat(hexCharacters, "]?");
var match6or8Hex = "#?[".concat(hexCharacters, "]{6}([").concat(hexCharacters, "]{2})?");
var nonHexChars = new RegExp("[^#".concat(hexCharacters, "]"), "gi");
var validHexSize = new RegExp("^".concat(match3or4Hex, "$|^").concat(match6or8Hex, "$"), "i");
/**
 * Convert HEX color to RGBA.
 * @param {string} hex The color in HEX format. Leading # is optional.
 * @param {Object} options
 * @param {string} options.format possible values: 'object' | 'array' | 'css'. Defaults to
 * 'object'. The RGB output format. Note that when using the css format, the value of the alpha
 * channel is rounded to two decimal places.
 * @param {number} options.alpha Set the alpha of the color. This overrides any existing alpha
 * component in the Hex color string. For example, the 99 in #22222299. The number must be in
 * the range 0 to 1.
 * @returns {Object | Array | string} An object, array or string containing the RGBA values.
 */

function hexToRgb(hex) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  if (typeof hex !== "string" || nonHexChars.test(hex) || !validHexSize.test(hex)) throw new TypeError("Expected a valid hex string");
  hex = hex.replace(/^#/, "");
  var alphaFromHex = 1;

  if (hex.length === 8) {
    alphaFromHex = Number.parseInt(hex.slice(6, 8), 16) / 255;
    hex = hex.slice(0, 6);
  }

  if (hex.length === 4) {
    alphaFromHex = Number.parseInt(hex.slice(3, 4).repeat(2), 16) / 255;
    hex = hex.slice(0, 3);
  }

  if (hex.length === 3) hex = hex[0] + hex[0] + hex[1] + hex[1] + hex[2] + hex[2];
  var number = Number.parseInt(hex, 16);
  var red = number >> 16;
  var green = number >> 8 & 255;
  var blue = number & 255;
  var alpha = typeof options.alpha === "number" ? options.alpha : alphaFromHex;
  if (options.format === "array") return [red, green, blue, alpha];

  if (options.format === "css") {
    var alphaString = alpha === 1 ? "" : " / ".concat(Number((alpha * 100).toFixed(2)), "%");
    return "rgb(".concat(red, " ").concat(green, " ").concat(blue).concat(alphaString, ")");
  }

  return {
    red: red,
    green: green,
    blue: blue,
    alpha: alpha
  };
}
/**
 * Convert RBG(A) colors to HEX format.
 * @param {number | string} red Either the full string in rgba format or a value for 'red'
 * between 0 and 255.
 * @param {number} green A value for 'green' between 0 and 255.
 * @param {number} blue A value for 'blue' between 0 and 255.
 * @param {number | string | undefined} alpha an optional alpha value as a number or percentage. 
 * @returns The HEX color.
 */


function rgbToHex(red, green, blue) {
  var alpha = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : undefined;
  var isPercent = (red + (alpha || "")).toString().includes("%");

  if (typeof red === "string") {
    var _red$match$map = red.match(/(0?\.?\d{1,3})%?\b/g).map(function (component) {
      return Number(component);
    });

    var _red$match$map2 = _slicedToArray(_red$match$map, 4);

    red = _red$match$map2[0];
    green = _red$match$map2[1];
    blue = _red$match$map2[2];
    alpha = _red$match$map2[3];
  } else if (alpha !== undefined) alpha = Number.parseFloat(alpha);

  if (typeof red !== "number" || typeof green !== "number" || typeof blue !== "number" || red > 255 || green > 255 || blue > 255) throw new TypeError("Expected three numbers below 256");

  if (typeof alpha === "number") {
    if (!isPercent && alpha >= 0 && alpha <= 1) alpha = Math.round(255 * alpha);else if (isPercent && alpha >= 0 && alpha <= 100) alpha = Math.round(255 * alpha / 100);else throw new TypeError("Expected alpha value (".concat(alpha, ") as a fraction or percentage"));
    alpha = (alpha | 1 << 8).toString(16).slice(1);
  } else alpha = "";

  return (blue | green << 8 | red << 16 | 1 << 24).toString(16).slice(1) + alpha;
}
//# sourceMappingURL=color.js.map