"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.is = is;
exports.isArrEqualDeep = isArrEqualDeep;
exports.isEqualShallow = isEqualShallow;
exports.isObjEqualDeep = isObjEqualDeep;
exports.isObjPropEqual = isObjPropEqual;
exports.objDiffShallow = objDiffShallow;
exports.partialShallowEqual = partialShallowEqual;

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

/**
 * Function implementing "Object.is" behaviour.
 * @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/is
 * @param {*} x The first value to compare.
 * @param {*} y The second value to compare.
 * @return {boolean} A boolean indicating whether or not the two arguments are the same value.
 */
function is(x, y) {
  if (x === y) {
    return x !== 0 || 1 / x === 1 / y;
  }

  return x !== x && y !== y;
}
/**
 * Checks whether a prop of an object compares equal in the other object (shallow comparison).
 * @param {Object} objA First object.
 * @param {Object} objB Second object.
 * @param {*} prop Property to compare.
 * @return {boolean} True if prop in objA is shallowly equal to prop in objB.
 */


function isObjPropEqual(objA, objB, prop) {
  return Object.prototype.hasOwnProperty.call(objB, prop) && is(objA[prop], objB[prop]);
}
/**
 * @see https://stackoverflow.com/questions/22266826/how-can-i-do-a-shallow-comparison-of-the-properties-of-two-objects-with-javascri#answer-37636728
 * @param {Object} objA First object.
 * @param {Object} objB Second object.
 * @return {boolean} True if the values of all keys are strictly equal.
 */


function isEqualShallow(objA, objB) {
  if (is(objA, objB)) return true;
  if (objA === null || objB === null || _typeof(objA) !== "object" || _typeof(objB) !== "object") return false;
  var keysA = Object.keys(objA);
  var keysB = Object.keys(objB);
  if (keysA.length !== keysB.length) return false; // Test for A's keys different from B.

  for (var i = 0; i < keysA.length; i++) {
    if (!isObjPropEqual(objA, objB, keysA[i])) return false;
  }

  return true;
}
/**
 * Compares two arrays deeply.
 * @param {Array} arrA First array.
 * @param {Array} arrB Second array.
 * @return {boolean} True if they are equal.
 */


function isArrEqualDeep(arrA, arrB) {
  if (arrA.length != arrB.length) return false;

  for (var i = 0; i < arrA.length; i++) {
    var a = arrA[i];
    var b = arrB[i];

    if (Array.isArray(a)) {
      if (!Array.isArray(b) || !isArrEqualDeep(a, b)) return false;
    } else if (_typeof(a) === "object") {
      if (_typeof(b) !== "object" || !isObjEqualDeep(a, b)) return false;
    } else if (!is(a, b)) return false;
  }

  return true;
}
/**
 * Compare two objects deeply.
 * @param {Object} objA First object.
 * @param {Object} objB Second object.
 * @return {boolean} True if they are equal by value or ref.
 */


function isObjEqualDeep(objA, objB) {
  if (Object.keys(objA).length !== Object.keys(objB).length) return false;

  for (var prop in objA) {
    var a = objA[prop],
        b = objB[prop];
    if (!b) return false;

    if (_typeof(a) === "object") {
      if (!_typeof(b) === "object" || !isObjEqualDeep(a, b)) return false;
    } else if (Array.isArray(a)) {
      if (!Array.isArray(b) || !isArrEqualDeep(a, b)) return false;
    } else if (a !== b) return false;
  }

  return true;
}
/**
 * Tests if a partial object is a subset of another object.
 * @param {Object} obj An object.
 * @param {Object} subset The object to check.
 * @return {boolean} False if subset has a key which is not in obj,
 * or has at least one key which is also in obj but with a different value
 * (weak comparison).
 */


function partialShallowEqual(obj, subset) {
  return isEqualShallow(Object.keys(subset).reduce(function (carry, key) {
    carry[key] = obj[key];
    return carry;
  }, {}), subset);
}
/**
 * If a property is on both objects but each object has a different value for that same property
 * (using weak equality comparison), the returned property will be set on both objects with their
 * respective values.
 * @return {Object} An object containing the shallow difference of the objects.
 */


function objDiffShallow(objA, objB) {
  var diff = {
    a: {},
    b: {}
  };
  if (isEqualShallow(objA, objB)) return diff;
  var keysA = Object.keys(objA);
  var keysB = Object.keys(objB);

  for (var i = 0; i < keysA.length; i++) {
    var prop = keysA[i];

    if (!isObjPropEqual(objA, objB, prop)) {
      diff.a[prop] = objA[prop];
      if (hasOwnProperty.call(objB, prop)) diff.b[prop] = objB[prop];
    }
  }

  for (var _i = 0; _i < keysB.length; _i++) {
    var _prop = keysB[_i];

    if (!isObjPropEqual(objB, objA, _prop)) {
      diff.b[_prop] = objB[_prop];
      if (hasOwnProperty.call(objA, _prop)) diff.a[_prop] = objA[_prop];
    }
  }

  return diff;
}
//# sourceMappingURL=compare.js.map