"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getNestedPropertyValue = getNestedPropertyValue;
exports.hasNestedPropertyValue = hasNestedPropertyValue;
exports.nestedMapGet = nestedMapGet;
exports.nestedMapHas = nestedMapHas;
exports.nestedMapSet = nestedMapSet;
exports.nestedObjectConstructValue = nestedObjectConstructValue;
exports.nestedTreeMapGet = nestedTreeMapGet;
exports.nestedTreeMapHas = nestedTreeMapHas;
exports.nestedTreeMapSet = nestedTreeMapSet;
exports.setNestedPropertyValue = setNestedPropertyValue;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

/**
 * Gets a nested value of an object given an array of nested property names (keys).
 * @param {Object} data JS POJO object.
 * @param {Array} props Array of keys in order of nesting.
 * @example
 * const source = { a: { b: { c: { d: 4 } } } };
 * getNestedPropertyValue(source, ["a", "b", "c", "d"]) === 4; // true
 * @return {*} The leaf value.
 */
function getNestedPropertyValue(data, props) {
  var root = data;

  for (var i = 0; i < props.length; i++) {
    var prop = props[i];
    root = root[prop];
  }

  return root;
}
/**
 * Checks if a nested value of an object given an array of nested property names (keys) exists.
 * @param {Object} data JS POJO object.
 * @param {Array} props Array of keys in order of nesting.
 * @return {boolean} True if the nested key exists, false otherwise.
 */


function hasNestedPropertyValue(data, props) {
  if (!props.length) return false;
  var root = data;

  for (var i = 0; i < props.length; i++) {
    var prop = props[i];
    if (!root[prop]) return false;
    root = root[prop];
  }

  return true;
}
/**
 * Sets a nested value of an object given an array of nested property names (keys).
 * @param {Object} data JS POJO object.
 * @param {Array} props Array of keys in order of nesting.
 * @param {*} value Leaf value.
 * @return {undefined}
 */


function setNestedPropertyValue(data, props, value) {
  if (!props.length) return;
  var root = data;
  var prev = null;

  for (var i = 0; i < props.length; i++) {
    var prop = props[i];
    if (_typeof(root[prop]) !== "object") root[prop] = {};
    prev = root;
    root = root[prop];
  }

  if (prev) prev[props[props.length - 1]] = value;
}
/**
 * Sets a nested value on a nested map.
 * @param {Map|WeakMap} map A map or weak map.
 * @param {Array} keys Array of keys to traverse. Each key will lead to a nested map.
 * @param {*} value The value to set at the inner key.
 * @return {undefined}
 */


function nestedMapSet(map, keys, value) {
  var i = 0;
  var current = map;

  while (i < keys.length - 1) {
    var key = keys[i];
    var nested = current.get(key);
    if (nested instanceof Map || nested instanceof WeakMap) current = nested;else {
      var newMap = new Map();
      current.set(key, newMap);
      current = newMap;
    }
    i++;
  }

  current.set(keys[i], value);
}
/**
 * Tests if a map has the given nested keys.
 * @param {Map|WeakMap} map A map or weak map.
 * @param {Array} keys Array of keys to check. Each key represents a nested map.
 * @return {boolean} "true" if all the nested keys exist, false otherwise.
 */


function nestedMapHas(map, keys) {
  var current = map;
  var i = 0;
  var l = keys.length;

  while ((current instanceof Map || current instanceof WeakMap) && current.has(keys[i]) && i < l) {
    current = current.get(keys[i]);
    i++;
  }

  return i == l;
}
/**
 * Gets a value from a nested map.
 * @param {Map|WeakMap} map A map or weak map.
 * @param {Array} keys Array of keys. Each key represents a nested map.
 * @return {*} The value of the map or "undefined" if there is no value for the given nested keys.
 */


function nestedMapGet(map, keys) {
  var current = map;
  var i = 0;
  var l = keys.length;

  while ((current instanceof Map || current instanceof WeakMap) && current.has(keys[i]) && i < l) {
    current = current.get(keys[i]);
    i++;
  }

  return i == l ? current : void 0;
}
/**
 * @type {Symbol}
 */


var treeMapSubtree = Symbol("treeMapSubtree");
/**
 * Sets a nested value on a nested tree map.
 * @param {Map|WeakMap} root A map or weak map to use as the root.
 * @param {Array} keys Array of keys to traverse. Each key will lead to a nested node of the tree map.
 * @param {*} value The value to set at the inner nested key.
 * @return {undefined}
 */

function nestedTreeMapSet(root, keys, value) {
  var i = 0;
  var current = root;
  var mapInstance = root instanceof WeakMap ? WeakMap : Map;

  while (i < keys.length - 1) {
    var _key = keys[i];
    var nested = current.get(_key);
    if (nested) current = nested[treeMapSubtree] || (nested[treeMapSubtree] = new mapInstance());else {
      var _node;

      var newMap = new mapInstance();
      var node = (_node = {}, _defineProperty(_node, treeMapSubtree, newMap), _defineProperty(_node, "value", void 0), _node);
      current.set(_key, node);
      current = newMap;
    }
    i++;
  }

  var key = keys[i];
  !current.has(key) ? current.set(key, {
    value: value
  }) : current.get(key).value = value;
}
/**
 * Tests if a tree map has the given nested keys.
 * @param {Map|WeakMap} rootMap The root of the map or weak map.
 * @param {Array} keys Array of keys to check. Each key represents a nested node of the tree map.
 * @return {boolean} "true" if all the nested keys exist, false otherwise.
 */


function nestedTreeMapHas(rootMap, keys) {
  var current = rootMap;
  var i = 0;
  var l = keys.length;

  while ((current instanceof Map || current instanceof WeakMap) && current.has(keys[i]) && i < l) {
    current = current.get(keys[i])[treeMapSubtree];
    i++;
  }

  return i == l;
}
/**
 * Gets a value from a nested tree map.
 * @param {Map|WeakMap} rootMap The root of the map or weak map.
 * @param {Array} keys Array of keys. Each key represents a nested node of the tree map.
 * @return {*} The value of the tree map or "undefined" if there is no value for the given nested keys.
 */


function nestedTreeMapGet(rootMap, keys) {
  var current = rootMap;
  var i = 0;
  var lastIndex = keys.length - 1;

  while ((current instanceof Map || current instanceof WeakMap) && current.has(keys[i]) && i < lastIndex) {
    current = current.get(keys[i])[treeMapSubtree];
    i++;
  }

  if (i === lastIndex && current) {
    var lastKey = keys[i];

    if (current.has(lastKey)) {
      var nested = current.get(lastKey);
      return nested.value;
    }
  }

  return void 0;
}
/**
 * Nests the properties of an object using an array of props definitions and defaults.
 * @param {Object} root The root object.
 * @param {Array} nestedPropsDef The nested props definitions.
 * @param {boolean} [arrOnFail] True if the root should be an array if the first argument
 * "root" is falsy.
 * @return {*} The leaf.
 */


function nestedObjectConstructValue(root, nestedPropsDef, arrOnFail) {
  root = root || (arrOnFail ? [] : {});
  var leaf = root;

  for (var i = 0; i < nestedPropsDef.length; i++) {
    var propDef = nestedPropsDef[i];
    var propKey = Object.keys(propDef)[0];
    var propDefault = propDef[propKey];
    leaf[propKey] = leaf[propKey] || propDefault;
    leaf = leaf[propKey];
  }

  return leaf;
}
//# sourceMappingURL=nested-properties.js.map