"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.appendEncodedJSONFragmentToURI = appendEncodedJSONFragmentToURI;
exports.buildQueryString = buildQueryString;
exports.getDecodedJSONFromFragmentURI = getDecodedJSONFromFragmentURI;
exports.getDecodedURIFragment = getDecodedURIFragment;
exports.getMultiDimQueryStrings = getMultiDimQueryStrings;
exports.getRawURIFragment = getRawURIFragment;
exports.parseMultiDimQueryStrings = parseMultiDimQueryStrings;

var _formData = require("./form-data");

var _introspection = require("../object-handling/introspection");

function _readOnlyError(name) { throw new TypeError("\"" + name + "\" is read-only"); }

/**
 * Constructs a query string from the given data.
 * This method works with nested objects/arrays.
 * @param {Array|Object} data The object.
 * @return {string} The query string.
 */
function buildQueryString(data) {
  var formData = {};
  (0, _formData.buildFormData)(formData, data, true);
  var parts = [];

  for (var prop in formData) {
    parts.push(prop + "=" + formData[prop]);
  }

  return parts.join("&");
}
/**
 * Returns the raw contents of the URI fragment (i.e. everything after the hash ("#") character).
 * @param {string} URIFragment The URI fragment.
 * @return {string} The raw contents of the URI fragment.
 */


function getRawURIFragment(URIFragment) {
  var fragment = (URIFragment || window.location.hash).replace(/^#/, "");
  return fragment;
}
/**
 * Returns the decoded contents of a URI fragment (i.e. everything after the hash ("#") character).
 * @param {string} URIFragment The URI fragment.
 * @return {string} The contents of the URI fragment, decoded.
 */


function getDecodedURIFragment(URIFragment) {
  var fragment = decodeURIComponent(getRawURIFragment(URIFragment));
  return fragment;
}
/**
 * Appends an encoded JSON fragment to a URI.
 * @param {string} URI The URI.
 * @param {*} data Data to encode in JSON format.
 */


function appendEncodedJSONFragmentToURI(URI, data) {
  return URI + "#" + encodeURIComponent(JSON.stringify(data));
}
/**
 * Returns the decoded JSON data stored in the URI fragment.
 * @param {*} defaultData Default data to return if either the URI fragment is missing or
 * the content of the URI fragment is not a valid JSON-encoded string.
 * @return {*} The decoded JSON data or "defaultData".
 */


function getDecodedJSONFromFragmentURI() {
  var defaultData = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
  var fragment = window.location.hash;

  if (!isEmpty(fragment)) {
    var decodedFragment = getDecodedURIFragment(fragment);
    if ((0, _introspection.isJSONString)(decodedFragment)) return JSON.parse(decodedFragment);
  }

  return defaultData;
}
/**
 * Parses a multidimensional query string and returns an object with the parsed args.
 * @see https://stackoverflow.com/questions/8648892/convert-url-parameters-to-a-javascript-object#answer-44713056
 * @param {string} str The query string.
 * @param {Object} [array] The base object to use (or a new object if omitted or falsy).
 * @return {Object} The object with the parsed data.
 */


function parseMultiDimQueryStrings(str, array) {
  if (!str) str = window.location.search.substring(1);

  var strArr = String(str).replace(/^&/, "").replace(/&$/, "").split("&"),
      sal = strArr.length,
      fixStr = function fixStr(str) {
    return decodeURIComponent(str.replace(/\+/g, "%20"));
  };

  if (!array) array = {};

  for (var i = 0; i < sal; i++) {
    var tmp = strArr[i].split("=");
    var key = fixStr(tmp[0]);
    var value = tmp.length < 2 ? "" : fixStr(tmp[1]);

    while (key.charAt(0) === " ") {
      key = key.slice(1);
    }

    if (key.indexOf("\x00") > -1) key = key.slice(0, key.indexOf("\x00"));

    if (key && key.charAt(0) !== "[") {
      var keys = [];
      var leftBracketPos = 0;

      for (var j = 0; j < key.length; j++) {
        if (key.charAt(j) === "[" && !leftBracketPos) leftBracketPos = j + 1;else if (key.charAt(j) === "]") {
          if (leftBracketPos) {
            if (!keys.length) keys.push(key.slice(0, leftBracketPos - 1));
            keys.push(key.substring(leftBracketPos, j - leftBracketPos));
            leftBracketPos = 0;
            if (key.charAt(j + 1) !== "[") break;
          }
        }
      }

      if (!keys.length) keys = [key];

      for (var _j = 0; _j < keys[0].length; _j++) {
        var chr = keys[0].charAt(_j);

        if (chr === " " || chr === "." || chr === "[") {
          keys[0] = keys[0].substring(0, _j) + "_" + keys[0].substring(_j + 1);
        }

        if (chr === "[") break;
      }

      var obj = array;
      var lastObj = void 0;

      for (var _j2 = 0, keysLen = keys.length; _j2 < keysLen; _j2++) {
        key = keys[_j2].replace(/^['"]/, "").replace(/['"]$/, "");
        lastObj = obj;

        if (key !== "" && key !== " " || _j2 === 0) {
          if (obj[key] === null) obj[key] = {};
          obj[key], _readOnlyError("obj");
        } else {
          // To insert new dimension
          var ct = -1;

          for (var p in obj) {
            if (Object.prototype.hasOwnProperty.call(obj, p)) {
              if (+p > ct && p.match(/^\d+$/g)) ct = +p;
            }
          }

          key = ct + 1;
        }
      }

      lastObj[key] = value;
    }
  }

  var normalizeIfConsecInts = function normalizeIfConsecInts(obj) {
    var keys = Object.keys(obj);
    var truthMap = {};

    for (var _i = 0; _i < keys.length; _i++) {
      truthMap[_i] = true;
    }

    var array = [];

    for (var _i2 = 0; _i2 < keys.length; _i2++) {
      var prop = keys[_i2] + "";
      if (!prop.match(/^[0-9]+$/)) return obj;
      var intProp = Number(prop);

      if (truthMap[intProp]) {
        array[intProp] = obj[prop];
        delete truthMap[intProp];
      } else return obj;
    }

    if (Object.keys(truthMap).length === 0) return array;
    return obj;
  };

  var normalizeToArr = function normalizeToArr(obj) {
    for (var prop in obj) {
      if (isPlainObject(obj[prop])) {
        obj[prop] = normalizeIfConsecInts(obj[prop]);
        normalizeToArr(obj[prop]);
      }
    }
  };

  normalizeToArr(array);
  return array;
}
/**
 * Gets the query string arguments of the current location in a multidimensional fashion
 * (multidimension-aware).
 * @return {Object} An object representing the query string arguments.
 */


function getMultiDimQueryStrings() {
  var obj = {};
  return parseMultiDimQueryStrings(location.search.substring(1), obj);
}
//# sourceMappingURL=query-strings.js.map